/**
 * importing components or modules
 */
import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { geoNaturalEarth1, geoPath, geoGraticule, json } from 'd3';
// import { json } from 'd3';
import { feature, mesh } from 'topojson';

//json data url
const jsonUrl = 'https://cdn.jsdelivr.net/npm/world-atlas@2/countries-110m.json';
/**
 * the following function extracts land features and interior countries mesh shapes
 * from json data, combines those land features and interior mesh and returns the data
 */
const useData = () => {
  const [data, setData] = useState(null);
  useEffect(() => {
    json(jsonUrl).then(topology => {
      const { countries, land } = topology.objects;
      setData({
        land: feature(topology, land),   //land features
        interiors: mesh(topology, countries, (a, b) => a !== b)  //interior countries mesh
      });
    });
  }, []);
  return data;
};

const projection = geoNaturalEarth1();   //geoNaturalEarth1() is used for geo projection
const path = geoPath(projection); //to generate the path or shape
const graticule = geoGraticule();  //to generate longitudes and latitudes

/**
 * Mark takes the data and combines the above defined projection, path and graticule
 * to form a full world map
 */
const Marks = ({ data: { land, interiors } }) => (
  <g className="marks">
    <path className="sphere" d={path({ type: 'Sphere' })} /> // for outline of globe
    <path className="graticules" d={path(graticule())} />  // langitudes and latitudes
    {land.features.map(feature => (				//generate land portions (green) of map
      <path className="land" d={path(feature)} />   
    ))}
    <path className="interiors" d={path(interiors)} />  // draw lines (dark green) to seperate countries
  </g>
);

const width = 960;
const height = 500;

// load the map into the App component
const App = () => {
  const data = useData();
  if (!data) {
    return <pre>Loading...</pre>;
  }
  return (
    // generate a svg and load the above defined features in the svg
    <svg width={width} height={height}>
      <Marks data={data} />
    </svg>
  );
};
const rootElement = document.getElementById('root');	//get the div with id "root" from html page
ReactDOM.render(<App />, rootElement);   // load the App component inside the selected div.
